@@ -4,12 +4,7 @@ Robot library with often needed keywords.
 
 """
 
-import datetime
-import os
-import random
-import time
-
-import robot
+import datetime, os, random, time, re, unicodedata, robot
 from SeleniumLibrary import LibraryComponent, JavaScriptKeywords, SelectElementKeywords, SeleniumLibrary
 from SeleniumLibrary.errors import ElementNotFound
 from SeleniumLibrary.utils import is_noney
@@ -166,7 +161,7 @@ class FacileToolkit2:
         self.set_selenium_library()
         self._wait_until(
             lambda: self.sl.find_element(locator).is_displayed() and self.sl.execute_javascript(JS_DOM_LOADED),
-            "Element '%s' or their parents did not contain class in <TIMEOUT>." % locator, timeout, error
+            "Element '%s' checkpoint failed in <TIMEOUT>." % locator, timeout, error
         )
 
     @keyword
@@ -276,7 +271,7 @@ class FacileToolkit2:
                                     x).is_displayed()]
         if not_visible_elements:
             errors = ', '.join(not_visible_elements)
-            BuiltIn().run_keyword_and_ignore_error("Should Be Empty", errors, "Following elements are not visible: %s" % errors)
+            BuiltIn().run_keyword_and_continue_on_failure("Should Be Empty", not_visible_elements, "Following elements are not visible: %s" % errors)
 
     def i_dont_see(self, *locators):
         """
@@ -289,7 +284,7 @@ class FacileToolkit2:
                                 x).is_displayed()]
         if visible_elements:
             errors = ', '.join(visible_elements)
-            BuiltIn().run_keyword_and_ignore_error("Should Be Empty", errors, "Following elements are visible: %s" % errors)
+            BuiltIn().run_keyword_and_continue_on_failure("Should Be Empty", visible_elements, "Following elements are visible: %s" % errors)
 
     @keyword(name="I Don't See")
     def i_don_t_see(self, *locators):
@@ -389,18 +384,26 @@ class FacileToolkit2:
         Checks if the given error message is present in DOM
         """
         self.set_selenium_library()
-        try:
-            self.sl.wait_until_element_is_visible(error_locator, timeout)
-            error_msgs = self.sl.get_webelements(error_locator)
-            found_errors = [el.text for el in error_msgs]
-            Collections().log_list(found_errors)
-            if is_truthy(match):
-                Collections().should_contain_match(found_errors, message, case_insensitive=is_truthy(case_insensitive))
-            else:
-                Collections().list_should_contain_value(found_errors, message)
-            logger.info("%s is contained in error message" % message)
-        except Exception as error_thrown:
-            raise Error(error_thrown)
+        self._wait_until(
+            lambda: self._error_message_should_be(message, error_locator, case_insensitive, match),
+            "Error finding message '%s' in <TIMEOUT>." % message, timeout, None
+        )
+
+    def _error_message_should_be(self, message, error_locator, case_insensitive, match):
+        """
+        Checks if the given error message is present in DOM
+        """
+        self.set_selenium_library()
+        self.sl.element_should_be_visible(error_locator)
+        error_msgs = self.sl.get_webelements(error_locator)
+        found_errors = [el.text for el in error_msgs]
+        Collections().log_list(found_errors)
+        if is_truthy(match):
+            Collections().should_contain_match(found_errors, message, case_insensitive=is_truthy(case_insensitive))
+        else:
+            Collections().list_should_contain_value(found_errors, message)
+        logger.info("%s is contained in error message" % message)
+        return True
 
     @keyword
     def wait_ajax(self, timeout=None, wait_active=None):
@@ -426,19 +429,19 @@ class FacileToolkit2:
 
     @keyword
     def field_or_parents_should_contain_error(self, locator, timeout=None, error_class="wrong", error=None):
-        self.wait_ultil_element_or_parents_contain_class(locator, error_class, timeout, error)
+        self.wait_until_element_or_parents_contain_class(locator, error_class, timeout, error)
 
     @keyword
     def field_or_parents_should_be_verified(self, locator, timeout=None, verified_class="verified", error=None):
-        self.wait_ultil_element_or_parents_contain_class(locator, verified_class, timeout, error)
+        self.wait_until_element_or_parents_contain_class(locator, verified_class, timeout, error)
 
     @keyword
     def field_should_contain_error(self, locator, timeout=None, error_class="wrong", error=None):
-        self.wait_ultil_element_contains_class(locator, error_class, timeout, error)
+        self.wait_until_element_contains_class(locator, error_class, timeout, error)
 
     @keyword
     def field_should_be_verified(self, locator, timeout=None, verified_class="verified", error=None):
-        self.wait_ultil_element_contains_class(locator, verified_class, timeout, error)
+        self.wait_until_element_contains_class(locator, verified_class, timeout, error)
 
     @keyword
     def fields_should_be_verified(self, *locators, timeout=None, verified_class="verified", error=None):
@@ -539,6 +542,7 @@ class FacileToolkit2:
         chrome_options.add_argument("--enable-features=NetworkService,NetworkServiceInProcess")
         chrome_options.add_argument("--no-sandbox")
         chrome_options.add_argument("--disable-gpu")
+        chrome_options.add_argument("--disable-dev-shm-usage")
         chrome_options.add_argument("--disable-browser-side-navigation")
         chrome_options.add_argument("----dns-prefetch-disable")
         chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
@@ -576,6 +580,7 @@ class FacileToolkit2:
         chrome_options.add_argument("--enable-features=NetworkService,NetworkServiceInProcess")
         chrome_options.add_argument("--no-sandbox")
         chrome_options.add_argument("--disable-gpu")
+        chrome_options.add_argument("--disable-dev-shm-usage")
         chrome_options.add_argument("--disable-browser-side-navigation")
         chrome_options.add_argument("----dns-prefetch-disable")
         chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
@@ -602,7 +607,7 @@ class FacileToolkit2:
         return date.strftime('%Y'), date.strftime('%m'), date.strftime('%d')
 
     @keyword
-    def wait_ultil_element_contains_class(self, locator, clazz, timeout=None, error=None):
+    def wait_until_element_contains_class(self, locator, clazz, timeout=None, error=None):
         self.set_selenium_library()
         self._wait_until(
             lambda: clazz in self.sl.get_element_attribute(locator, "class"),
@@ -611,7 +616,7 @@ class FacileToolkit2:
         )
 
     @keyword
-    def wait_ultil_element_does_not_contain_class(self, locator, clazz, timeout=None, error=None):
+    def wait_until_element_does_not_contain_class(self, locator, clazz, timeout=None, error=None):
         self.set_selenium_library()
         self._wait_until(
             lambda: clazz not in self.sl.get_element_attribute(locator, "class"),
@@ -626,7 +631,7 @@ class FacileToolkit2:
         return self.sl.driver.execute_script(get_parents, self.sl.find_element(locator))
 
     @keyword
-    def wait_ultil_element_or_parents_contain_class(self, locator, clazz, timeout=None, error=None):
+    def wait_until_element_or_parents_contain_class(self, locator, clazz, timeout=None, error=None):
         elements = self.get_element_parents(locator)
         self._wait_until(
             lambda: any(clazz in element.get_attribute("class") for element in elements),
@@ -635,7 +640,7 @@ class FacileToolkit2:
         )
 
     @keyword
-    def wait_ultil_element_or_parents_do_not_contain_class(self, locator, clazz, timeout=None, error=None):
+    def wait_until_element_or_parents_do_not_contain_class(self, locator, clazz, timeout=None, error=None):
         elements = self.get_element_parents(locator)
         self._wait_until(
             lambda: any(clazz not in element.get_attribute("class") for element in elements),
@@ -754,7 +759,7 @@ class FacileToolkit2:
         self.set_selenium_library()
         mandatory = is_truthy(mandatory)
         try:
-            self.sl.wait_until_element_is_visible('css:#onetrust-banner-sdk[style*="0px"]', timeout)
+            self.sl.wait_until_element_is_visible('css:#onetrust-banner-sdk[style*="0px"],#onetrust-banner-sdk:not([style])', timeout)
         except:
             if mandatory:
                 raise
@@ -764,6 +769,22 @@ class FacileToolkit2:
             if mandatory:
                 raise
 
+    @keyword
+    def slugify(self, value, allow_unicode=False):
+        """
+        Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
+        dashes to single dashes. Remove characters that aren't alphanumerics,
+        underscores, or hyphens. Convert to lowercase. Also strip leading and
+        trailing whitespace, dashes, and underscores.
+        """
+        value = str(value)
+        if allow_unicode:
+            value = unicodedata.normalize('NFKC', value)
+        else:
+            value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
+        value = re.sub(r'[^\w\s-]', '', value.lower())
+        return re.sub(r'[-\s]+', '-', value).strip('-_')
+
     def _wait_until(self, condition, error, timeout=None, custom_error=None):
         self.set_selenium_library()
         timeout = self.lc.get_timeout(timeout)
@@ -786,6 +807,8 @@ class FacileToolkit2:
             except StaleElementReferenceException as err:
                 self.lc.info('Suppressing StaleElementReferenceException from Selenium.')
                 not_found = err
+            except AssertionError as err:
+                not_found = None
             else:
                 not_found = None
             time.sleep(0.2)
